/***************************************************************************
                          CColorBar.cpp  -  description
                             -------------------
    begin                : Sun Mar 26 2000
    copyright            : (C) 2000 by Alexander Theel
    email                : alex.theel@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../../global.h"
#include "CColorBar.h"

#include <qpainter.h>
#include <qfontmetrics.h>

#include <iostream>

/**
 * Default-constructor with default values for the colors and the strings.
 * the properties should be reset with 'change(..)'
 */
// -------------------------------------------------------------------------------
CColorBar::CColorBar( QWidget* pParent, const QString& sName )
 : QWidget( pParent, sName )
 , mColorTop( 123,34,76 )
 , mColorBottom( 30,30,30 )
 , mFontColor( white )
 , msTextTop( "Test" )
 , msTextBottom( "Bar" )
 , msVerticalText( "" )
 , mbAlignVerticalText( ALIGN_BOTTOM )
// -------------------------------------------------------------------------------
{
}

// -------------------------------------------------------------------------------
CColorBar::CColorBar( QWidget* pParent,  const QColor& c1,  const QColor& c2,
                    const QString& t1, const QString& t2, const QColor& c3 )
 : QWidget( pParent )
 , mColorTop( c1 )
 , mColorBottom( c2 )
 , mFontColor( c3 )
 , msTextTop( t1 )
 , msTextBottom( t2 )
 , msVerticalText( "" )
 , mbAlignVerticalText( ALIGN_BOTTOM )
// -------------------------------------------------------------------------------
{
   setMinimumSize(33, 200);
}


// -------------------------------------------------------------------------------
void CColorBar::change( const QColor& c1,  const QColor& c2,
                       const QString& t1, const QString& t2, const QColor& c3 )

// -------------------------------------------------------------------------------
{
   mColorTop    = c1;
   mColorBottom = c2;
   mFontColor   = c3;

   msTextTop    = t1;
   msTextBottom = t2;

   repaint();
}



// dummy method
// -------------------------------------------------------------------------------
void CColorBar::setFlowerDirectory( const QString& )
// -------------------------------------------------------------------------------
{}



/**
 * lets the user set a vertical Text; this one must be given in 'text'.
 * the argument 'top_bottom' indicates wether 'text' is aligned to the
 * top(=false) of the colorbar or to the bottom(=true)
 */
// -------------------------------------------------------------------------------
void CColorBar::setVerticalText( const QString& sText, bool bAlign )
// -------------------------------------------------------------------------------
{
   msVerticalText      = sText;
   mbAlignVerticalText = bAlign;
}


// -------------------------------------------------------------------------------
void CColorBar::paintEvent( QPaintEvent* )
// -------------------------------------------------------------------------------
{
   QPainter p;                           // our painter
   p.begin(this);                        // start painting widget
      paint(p);
   p.end();                              // painting done
}


// -------------------------------------------------------------------------------
void CColorBar::paint( QPainter& p )
// -------------------------------------------------------------------------------
{
   // calculate width
   QFont f=p.font(); f.setBold(true);
   QFontMetrics fm(f);
   int widthT1=fm.width(msTextTop), widthT2=fm.width(msTextBottom);
   int w;
   if(widthT1>widthT2) w=widthT1; else w=widthT2;
   if(w<VERTICAL_TEXT_SIZE) w=VERTICAL_TEXT_SIZE+5;
   setMinimumWidth(w);
   p.setFont(f);

   int h=height();

   // define BasisColor & DestinationColor
   int rBasis=mColorTop.red(),  gBasis=mColorTop.green(),  bBasis=mColorTop.blue();
   int rDest=mColorBottom.red(),gDest=mColorBottom.green(),bDest=mColorBottom.blue();

   // difference between BasisColor & DestinationColor
   int rDiff=rDest-rBasis;
   int gDiff=gDest-gBasis;
   int bDiff=bDest-bBasis;

   int stepSize=4;
   int y;
   for (y=0; y<h; y+=stepSize)
   {
      QColor color(rBasis+y*rDiff/h, gBasis+y*gDiff/h, bBasis+y*bDiff/h);
      QBrush b(color);
      p.fillRect(0,y, w,y+stepSize, b);
   }

   // writing
   p.setPen( mFontColor );
   p.drawText(w/2-widthT1/2, h-30, msTextTop);
   p.drawText(w/2-widthT2/2, h-15, msTextBottom);

   // vertical Text
   if(msVerticalText!="")
   {
      f.setBold(true); f.setPointSize(VERTICAL_TEXT_SIZE); p.setFont(f);
      QFontMetrics fm(f);

      int width=fm.width(msVerticalText);
      int x = (mbAlignVerticalText==ALIGN_TOP ? x = -1 * width : x = -1 * this->height() );

      p.rotate(-90);
      p.drawText(x, this->width()/2 + fm.height()/2-3, msVerticalText);
      p.rotate(90);                       // rotate back for the other drawing-things
   }
}
